package ida.pelab.cgn.ccode;



import ida.pelab.prt.pattern.ArrayBoundary;
import ida.pelab.prt.pattern.LoopVariable;
import ida.pelab.prt.pattern.SubscriptedIndexList;
import ida.pelab.prt.pattern.SubscriptedIndexTree;
import ida.pelab.prt.transformation.ExpressionSimplifier;


import java.util.ArrayList;










import cetus.hir.AnnotationDeclaration;
import cetus.hir.ArrayAccess;


import cetus.hir.CodeAnnotation;


import cetus.hir.Identifier;
import cetus.hir.Specifier;
import cetus.hir.Symbol;

import cetus.hir.TranslationUnit;
import cetus.hir.Traversable;




public class CodeExtension {

	

 
  public CodeExtension()
  {
	
  }
  
  


  /**
   * generates a unique variable for in the output which corresponds to the loop variable of the current for-loop
 * @param subTreeRoot
 * @param lv
 * @return
 */
public String addLoopVariable(Traversable subTreeRoot,LoopVariable lv)
  {
	  	  String variableString= "   struct range rng_"+ getNodeName(lv.getVariableExpression())
	  	  +" ={ " + ".lowerBound=" +ExpressionSimplifier.Simplify( lv.getLowerBound()).toString()+ ", .upperBound=" +
	  	ExpressionSimplifier.Simplify(lv.getUpperBound()).toString() + ", .step= " + ExpressionSimplifier.Simplify(lv.getStep()).toString()+ "};\n";
	  	  return variableString;

  }
  
  public String addVectorStatement(Traversable arrayNode,ArrayList<ArrayBoundary> boundaryList)
  {
	  if(boundaryList.size()<1)
		  return "boundary not valid";
	  ArrayBoundary boundary=boundaryList.get(0);
	  String vectorString= getDimensionArray((ArrayAccess)arrayNode);
	  vectorString+= "   struct vector vec_"+ getNodeName(arrayNode)
	  +"={\n     .source={.ptr= "+((ArrayAccess)arrayNode).getArrayName()
	  +",  .type=" + getArrayType(arrayNode)+", .dimension=5} ,"
	  +" \n     .subscriptedDimension=" +getSubscriptedDimension(arrayNode,1)+",\n     .boundary= {"
	  +".lowerBound="+ boundary.getLowerBound()+" ,.upperBound=" + boundary.getUpperBound()
	  +",.step="+ boundary.getStride()+"}\n    };\n";
	  vectorString+= getDimensionInitialStatement(arrayNode);
	  return vectorString;
	  
  }
  
  public String addMatrixStatement(Traversable arrayNode,ArrayList<ArrayBoundary> boundaryList)
  {
	  if(boundaryList.size()<1)
		  return "boundary not valid";
	  ArrayBoundary firstBnd=boundaryList.get(0);
	  ArrayBoundary secBnd=boundaryList.get(1);
	  String vectorString= getDimensionArray((ArrayAccess)arrayNode);
	  vectorString+= "   struct matrix mtrx_"+ getNodeName(arrayNode)
	  +"={\n     .source={.ptr= "+((ArrayAccess)arrayNode).getArrayName()
	  +",  .type=" + getArrayType(arrayNode)+", .dimension=5} ,"
	  +" \n     .fstSubscriptedDim;=" +getSubscriptedDimension(arrayNode,1)+",\n"
	  +" \n     .secSubscriptedDim;=" +getSubscriptedDimension(arrayNode,2)+",\n     .fstBoundary= {"
	  +".lowerBound="+ firstBnd.getLowerBound()+" ,.upperBound=" + firstBnd.getUpperBound()
	  +",.step="+ firstBnd.getStride()+"}"+
	  "\n     .secBoundary= {"
	  +".lowerBound="+ secBnd.getLowerBound()+" ,.upperBound=" + secBnd.getUpperBound()
	  +",.step="+ secBnd.getStride()+"}"+
	  "\n    };\n";
	  vectorString+= getDimensionInitialStatement(arrayNode);
	  return vectorString;
	  
  }
  private String getDimensionInitialStatement(Traversable arrayNode)
  {
	  String vectorName="vtr_"+ getNodeName(arrayNode);
	  String dimArrayName="dim" +((ArrayAccess)arrayNode).getArrayName();
	  return "   array_copydimension(& " +vectorName+".source,"+dimArrayName+","
	  +Integer.toString(((ArrayAccess)arrayNode).getIndices().size())+" ); \n\n";
	  
  }
  private String getDimensionArray(ArrayAccess array)
  {
	  String dimensionString= "   int dim"+array.getArrayName() +"["+ 
	  Integer.toString(array.getIndices().size()) +"]={";
	  for(int i=0;i<array.getIndices().size();i++)	  
		 dimensionString+=   array.getIndex(i).toString()+",";
	  dimensionString=dimensionString.substring(0,dimensionString.lastIndexOf(","));
	  dimensionString+="}; \n" ;

	  return dimensionString;
  }
  private String getSubscriptedDimension( Traversable arrayNode,int index)
  {
	  SubscriptedIndexList indexList= SubscriptedIndexTree.getIndexList(arrayNode);
	  if(indexList.getSubscriptedList().size()<1)
		  return "-1";
	  
	  int subscriptedIndex=indexList.getSubscriptedList().size()-index;
	  return Integer.toString(indexList.getSubscriptedList().get(subscriptedIndex));
  }
  private String getArrayType(Traversable arrayNode)
  {
	   Identifier arrayName=(Identifier)((ArrayAccess)arrayNode).getArrayName();
	   Symbol eqSymb=arrayName.getSymbol();
	   if(eqSymb==null)
		   return "-1";
	   java.util.List<?>  typeList= eqSymb.getTypeSpecifiers();
	   
	   if(typeList!=null)
	    for(int i=0; i< typeList.size();i++)
	    	if(typeList.get(i) instanceof Specifier)
	    	{
	    		Specifier typeCast=(Specifier)typeList.get(i);
	    		if(typeCast.equals(Specifier.INT))
	    			return "1";
	    		if(typeCast.equals(Specifier.FLOAT))
	    			return "2";
	    		if(typeCast.equals(Specifier.DOUBLE))
	    			return "3";
	    		if(typeCast.equals(Specifier.LONG))
	    			return "4";
	    		
	    	}
	 
	    //invalid type;
	  return "-1";
	  
  }


  public String getNodeName(Traversable node)
  {
	  return Integer.toString( System.identityHashCode(node));
  }
      
     public static void addIncludeStatements(Traversable rootNode,ArrayList<String> directiveLists)
     {
    	 if(!(rootNode.getChildren().get(0) instanceof TranslationUnit))
    		 return;
    	 TranslationUnit transUnit=(TranslationUnit)rootNode.getChildren().get(0);
    	 for(int i=0; i< directiveLists.size();i++)
    	 {
    		 AnnotationDeclaration includeDec= new AnnotationDeclaration(new CodeAnnotation("#include \"" + directiveLists.get(i) +"\""));
    		 transUnit.addDeclarationFirst( includeDec);
    	 }
  
    	 
     }
    

     public void generateWrapperFile()
     {
    	 
     }
  
  
  
}
