package ida.pelab.loopdistributionTool;

import java.util.ArrayList;




import cetus.analysis.DDGraph;
import cetus.analysis.DDTDriver;


import cetus.analysis.DDGraph.Arc;

import cetus.hir.*;


public class LoopDistributor {
	
	
	
  private static  DDTDriver dependencyDriver;
  private static Program  sourceProgram;
  private static boolean  distributionFlag=false;
	
  //applies loop distribution on a certain sub-tree by generating new loops and extending
  // the IR structure for this sub-tree
	public static void applyForLoopDistribution(Traversable rootNode)
	{
		
		 sourceProgram =(Program)rootNode;
		 callDependencyAnalysis();
		 if( !distributionFlag)
			 return;
		PostOrderIterator<Traversable> iterator = new PostOrderIterator<Traversable>(rootNode);
		while(iterator.hasNext())
		{
			Object  currentNode =iterator.next();
			if(currentNode instanceof ForLoop)			
				{
				   distributeForLoop((ForLoop)currentNode);
				
				}
				
			
		}
		
	}
		
	
	// Invokes the Data dependency driver of Cetus which finds the subscripted data dependencies
	// among statements
	private static void callDependencyAnalysis()
	{
		try
		{
			
			dependencyDriver= new DDTDriver(sourceProgram);
			dependencyDriver.start();	
			 distributionFlag=true ;

		}
		
		catch (ClassCastException e) {
			//	e.printStackTrace();

				System.err.println("loop anaylsis is not supported for the current code block in this version of Cetus");
				System.err.println("Skipping Loop distribution....");
				distributionFlag=false;
				return;
				
			}
			
			catch (NullPointerException e) {
			//	e.printStackTrace();

				System.err.println("Error during loop analysis, make sure all variables used in the program are defined");
				System.err.println("Skipping Loop distribution....");
				distributionFlag=false;
				return ;
				
			}
			
			catch (Exception e) {
				
				// cetus can't handle anlysis in the following situations :
					//there is a while or do while block inside the  code block
					// the definition of the variables used in the section is not found ;
				//e.printStackTrace();
				
				System.err.println(e.getMessage().toString());
				System.err.println("Error during loop analysis");
				System.err.println("Skipping Loop distribution....");
				distributionFlag=false; 
				return;
			}
		
	}
	

	
	@Deprecated
	private static void printDependencies(DDGraph dg, ArrayList<Arc> dependencies)
	{
		

	
		
		for(int k= 0; k<dependencies.size();k++)
			{
			     Arc f =dependencies.get(k);
			     if(f.getSinkStatement()!=f.getSourceStatement() || 1==1  )
			     {
			    	 System.out.println(f.toString());
			    	 System.out.print("    SourceExp=       " +f.getSource().getArrayAccess().toString());			     
			    	 System.out.println("    SinKEXP=       " +f.getSink().getArrayAccess().toString());			    
			    	 System.out.println("    source=        "+f.getSourceStatement().toString());
			    	 System.out.println("    sink=       " +f.getSinkStatement().toString());		    	
			    	 System.out.println("    Dependency Vector"  +  f.getDependenceVector().VectorToString());
	
			    	 
			     }
		
			
			}
		System.out.println("''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''");
		
		
		
	}
	private  static void distributeForLoop(ForLoop currentLoop)
	{
		
		DDGraph dependencyGraph =null;
						
		//dependency analysis for subscripted variables in the body
		try{
			
		
	     dependencyGraph  =dependencyDriver.analyzeLoopsForDependence(currentLoop);
		}
	 	catch (ClassCastException e) {
			//	e.printStackTrace();

				System.err.println("loop anaylsis is not supported for the current code block in this version of Cetus");
				System.err.println("Skipping Loop distribution....");
				distributionFlag=false;
				return ;
				
			}
	
	    // getting the list of dependency arcs 
		ArrayList<Arc> dependencyArcs = dependencyGraph.getAllArcs();
	
		// prunning the set of dependencies which are not important for loop distribution
		// i.e dependencies from a statement to itself
		dependencyArcs =pruneDependencies(dependencyArcs);
	
		//printDependencies(dependencyGraph,dependencyArcs);
		
		// Dependency Analysis for scalar variables :
		 ScalarDependency scalarDependencyAnalysis = new ScalarDependency();
		
		 ArrayList<Dependency> dependencyList =scalarDependencyAnalysis.getDependencyListInLoop(currentLoop);
		  
		// printDependencies(dependencyGraph,dependencyArcs);
		  if(dependencyArcs.size()==0 && dependencyList.size()==0 && ((CompoundStatement)currentLoop.getBody()).getChildren().size()<2)
			  return ;
		 
		  DirectedGraph  pathGraph = new  DirectedGraph();
		  pathGraph.ConstructFromForLoop(currentLoop,dependencyArcs, dependencyList);
		  
		  // finding the strongly connected components 
		  ArrayList<ArrayList<Traversable>> nodeGroups= TarjanAlgorithm.findStronglyConnectedComponents(pathGraph);
		  
		  //sorting the components by the execution order		 
		  if(nodeGroups.size()>1)
		    {
			  nodeGroups=sortByExectutionOrder(nodeGroups,currentLoop);
			  generateForHeader(currentLoop, nodeGroups);
			  callDependencyAnalysis();
		    }
		 
	    
		
	}
	
	
	private static ArrayList<ArrayList<Traversable>> sortByExectutionOrder(ArrayList<ArrayList<Traversable>> nodeGroups,Loop order)
	{
		for(int i= 0 ; i<nodeGroups.size();i++)
			nodeGroups.set(i, MergeSort.sort(nodeGroups.get(i)));
		
		nodeGroups=MergeSort.sort(nodeGroups);
		
		return nodeGroups;
		
	}
	
	
	
	private static void generateForHeader(ForLoop signatureForLoop, ArrayList<ArrayList<Traversable>>  nodeGroups )
	{
		
		for(int i= 0;i<nodeGroups.size();i++)
		{
			Statement newBody = signatureForLoop.getBody().clone();
			newBody.getChildren().clear();
			newBody.getChildren().addAll(nodeGroups.get(i));
			for(int j = 0 ; j<nodeGroups.get(i).size();j++)
			  nodeGroups.get(i).get(j).setParent(newBody);
			ForLoop newLoop =new ForLoop(signatureForLoop.getInitialStatement().clone(), signatureForLoop.getCondition().clone(),signatureForLoop.getStep().clone(), newBody);	 	
			CompoundStatement parentBody=(CompoundStatement)signatureForLoop.getParent();
 		 	parentBody.addStatementBefore(signatureForLoop, newLoop);
 		 	ScalarDependency  analysis = new ScalarDependency();
 		 	// analysing ScalarAnalysis for each new loop;
 		 	analysis.getDependencyListInLoop(newLoop);
		
		}
		CompoundStatement parentBody=(CompoundStatement)signatureForLoop.getParent();
		parentBody.removeChild(signatureForLoop);
		
		
	}
	
	//checks whether a dependency is backward or forward
	 public static  boolean isBackwardDependency(Arc dependency,Traversable parentLevel )
	 {	 
		 Statement source = dependency.getSourceStatement();
		 Statement sink =dependency.getSinkStatement() ;
         return isAfter(source, sink, parentLevel);

	 }
	 
	 private  static boolean isAfter(Statement source, Statement sink ,Traversable nodelevel)
	 {
		 Traversable  sourceLevelnode =getEquivalentNodeLevel(source, nodelevel);
		 Traversable  sinkLevelnode =getEquivalentNodeLevel(sink, nodelevel);
		 
		 return nodelevel.getChildren().indexOf(sourceLevelnode)>nodelevel.getChildren().indexOf(sinkLevelnode);
		 
		 
	 }
	 public  static Traversable getEquivalentNodeLevel(Statement currentNode,Traversable parentLevel)
	 {
		 Traversable node =(Traversable) currentNode;
		 while(node.getParent()!=parentLevel )
			 node =node.getParent();

		return node ; 
	 }
	 

	 
	 // The list of dependencies from any statment to itself is removed ,as they are not important
	 // for the loop distribution process
	 private static ArrayList<Arc>  pruneDependencies(ArrayList<Arc> dependencyList)
	 {
		 
		
			for(int i = 0 ; i<dependencyList.size();i++)
			   if (dependencyList.get(i).getSinkStatement()==dependencyList.get(i).getSourceStatement())
			   	{  
				    dependencyList.remove(i);
					i--;
				}
				
			return dependencyList;		
		 
	 }
	 


}
