package ida.pelab.loopdistributionTool;


import ida.pelab.prt.generalfunctions.IgnoredNodes;

import java.util.ArrayList;




import cetus.hir.ArrayAccess;
import cetus.hir.AssignmentExpression;
import cetus.hir.BinaryExpression;

import cetus.hir.Case;
import cetus.hir.CommaExpression;
import cetus.hir.CompoundStatement;

import cetus.hir.Expression;
import cetus.hir.ExpressionStatement;

import cetus.hir.FunctionCall;
import cetus.hir.Identifier;
import cetus.hir.IfStatement;
import cetus.hir.Loop;
import cetus.hir.ReturnStatement;
import cetus.hir.Statement;
import cetus.hir.SwitchStatement;
import cetus.hir.Traversable;
import cetus.hir.Typecast;
import cetus.hir.UnaryExpression;


public class ScalarDependency {
	

	
	public ScalarDependency()
	 {

	 }
	
	public  ArrayList<Expression> getDefinitionList(Statement  stmt)
	{
				
		ArrayList<Expression> outPutList = new ArrayList<Expression>();
		  
		 if(stmt instanceof ExpressionStatement    && 
		 ((ExpressionStatement)stmt).getExpression() instanceof AssignmentExpression		 
		  )
		 {
			 AssignmentExpression assignment = (AssignmentExpression)((ExpressionStatement)stmt).getExpression();
		     //since we are only interested in scalar variables	
			 if(assignment.getLHS() instanceof Identifier)
				 outPutList.add(assignment.getLHS());		 
		 }
		 
		 
		//covers cases like ++j,--j,j++,i++
		 if(stmt instanceof ExpressionStatement    && 
				 ((ExpressionStatement)stmt).getExpression() instanceof UnaryExpression		 
				  )
		 {
			 UnaryExpression uExpression = (UnaryExpression)((ExpressionStatement)stmt).getExpression();
			 outPutList.add(uExpression.getExpression());
			 
		 }
		 
		return outPutList;
		
	}
	
	private ArrayList<Expression>  extractExpression(Traversable node)
	{
		
		ArrayList<Expression> list = new ArrayList<Expression>();
	    if(node instanceof Identifier)
	    	list.add((Expression)node);
	    else if(node instanceof ArrayAccess)
	    {
	    	ArrayAccess array=(ArrayAccess)node ;
	     //since we are only interested in scalar variables	the array itself is not added to the result set
	    //	list.add((Expression)array);
	    	for(int i = 0 ; i<array.getIndices().size();i++)
	    		list.addAll(extractExpression(array.getIndex(i)));
	    	
	    }
	    else if(node instanceof BinaryExpression)
	    {
	    	list.addAll(extractExpression(((BinaryExpression)node).getLHS()));
	    	list.addAll(extractExpression(((BinaryExpression)node).getRHS()));
	    	
	    }
	    else if(node instanceof CommaExpression)
	    {
	    	CommaExpression commaExp =(CommaExpression)node ;
	    	for(int i = 0 ; i<commaExp.getChildren().size();i++)
	    		list.addAll(extractExpression(commaExp.getChildren().get(i)));
	    		
	    }
	    else if (node instanceof Typecast)
	    {
	    	Typecast typeCasting =(Typecast) node ;
	    	list.addAll(extractExpression(typeCasting.getExpression()));

	    }
	    else if (node instanceof FunctionCall)
	    {
	    	FunctionCall funCall= (FunctionCall)node ;
	    	for(int i = 0 ; i< funCall.getArguments().size();i++ )
	    		list.addAll(extractExpression(funCall.getArgument(i)));
	    	
	    }
	    
	    else if (node instanceof UnaryExpression)
	    {
	    	UnaryExpression unExp= (UnaryExpression)node ;	    	
	    		list.addAll(extractExpression(unExp.getExpression()));
	    	
	    }
		return list;
		
	}
	public  ArrayList<Expression> getUseList(Statement  stmt)
	{
				
		ArrayList<Expression> outPutList = new ArrayList<Expression>();
		  
		 if(stmt instanceof ExpressionStatement    && 
		 ((ExpressionStatement)stmt).getExpression() instanceof AssignmentExpression		 
		  )
		 {
			 AssignmentExpression assignment = (AssignmentExpression)((ExpressionStatement)stmt).getExpression();
			 outPutList.addAll(extractExpression(assignment.getRHS()));
			 if(assignment.getLHS() instanceof ArrayAccess)
				 outPutList.addAll(extractExpression(assignment.getLHS()));
			 
		 }		
		 else if(stmt instanceof ExpressionStatement    && 
				 ((ExpressionStatement)stmt).getExpression() instanceof FunctionCall		 
		  )
			 outPutList.addAll(extractExpression((FunctionCall)((ExpressionStatement)stmt).getExpression()));

		
		 if(stmt instanceof ReturnStatement)
		 {
			 ReturnStatement rt =(ReturnStatement) stmt;
		       return getUseExpression(rt.getExpression());

		 }
		
		 
		 if(stmt instanceof ExpressionStatement    && 
				 ((ExpressionStatement)stmt).getExpression() instanceof UnaryExpression		 
				  )
		 {
			 UnaryExpression uExpression = (UnaryExpression)((ExpressionStatement)stmt).getExpression();
			 outPutList.add(uExpression.getExpression());
			 
		 }
			
		return outPutList;
		
	}
	
	public  ArrayList<Expression> getUseExpression(Expression exp)
	{
				
		ArrayList<Expression> outPutList = new ArrayList<Expression>();
		outPutList.addAll(extractExpression(exp));

		return outPutList;
		
	}
	
	// dependencies in both direction
	public ArrayList<Dependency> getDependencyList(Traversable firstNode ,Traversable secondNode)
	{
		ElementSet firstSet= getDefUseSet(firstNode);

		ElementSet secondSet= getDefUseSet(secondNode);
		ArrayList<Expression> defStmt1 =firstSet.getDefinitionSet();
		ArrayList<Expression> defStmt2 =secondSet.getDefinitionSet();
		ArrayList<Expression> useStmt1 =firstSet.getUseSet();
		ArrayList<Expression> useStmt2 =secondSet.getUseSet();
		
		ArrayList<Dependency> dependencyList = new ArrayList<Dependency>();
		
	    for(int i = 0 ; i<defStmt1.size();i++)
	    {	    	
	    	Expression currentExp= defStmt1.get(i);	    	
	    	if(defStmt2.indexOf(currentExp)>-1)
	    		{
	    			
	    			dependencyList.add(new Dependency(currentExp, firstNode, defStmt2.get(defStmt2.indexOf(currentExp)), secondNode, DependenceType.Output));
	    			dependencyList.add(new Dependency(defStmt2.get(defStmt2.indexOf(currentExp)), secondNode,currentExp, firstNode, DependenceType.Output));
	    		}
	    	if(useStmt2.indexOf(currentExp)>-1)
	    		{
	    			dependencyList.add(new Dependency(currentExp, firstNode, useStmt2.get(useStmt2.indexOf(currentExp)), secondNode, DependenceType.Flow));
	    			dependencyList.add(new Dependency(useStmt2.get(useStmt2.indexOf(currentExp)), secondNode,currentExp, firstNode, DependenceType.Anti));
	    		}
	    	
	    	
	    }
	    
	    for(int i = 0 ; i<useStmt1.size();i++)
	    {
	    	Expression currentExp= useStmt1.get(i);	   
	    	if(defStmt2.indexOf(currentExp)>-1)
	    	{
	    		dependencyList.add(new Dependency(currentExp, firstNode, defStmt2.get(defStmt2.indexOf(currentExp)), secondNode, DependenceType.Anti));
	    		dependencyList.add(new Dependency( defStmt2.get(defStmt2.indexOf(currentExp)), secondNode,currentExp, firstNode, DependenceType.Flow));
	    		
	    		
	    	}

	    	
	    	
	    }
		
		return dependencyList;
	}

	
	public ElementSet getDefUseSet(Traversable node )
	{
		ElementSet resultSet=new ElementSet();
		
		if(node instanceof Loop)
		{
			resultSet =DependencyPool.getDefUseSet(node);
			if(resultSet==null)
				{
					getDependencyListInLoop((Loop)node);
					resultSet=DependencyPool.getDefUseSet(node);
				}
		
			
		}
		
		else if (node instanceof IfStatement)
		{
		
		resultSet=	DependencyPool.getDefUseSet(node);
			if(resultSet==null)
				{
					getDependencyListInIF((IfStatement)node);
					resultSet=DependencyPool.getDefUseSet(node);
			     }
		  }	
		else if (node instanceof SwitchStatement)
		{
			SwitchStatement swch=(SwitchStatement) node ;
			ArrayList<Expression> definitionSet= new ArrayList<Expression>();
			ArrayList<Expression> useSet =getUseExpression(swch.getExpression());
			resultSet.setDefSet(definitionSet);
			resultSet.setUseSet(useSet);
		}
		else if (node instanceof Case)
		{
			//Case caseNode =(Case)node ;
			//caseNode.
		}

		else if(node instanceof Statement)
		{
			ArrayList<Expression> definitionSet= getDefinitionList((Statement)node);
			ArrayList<Expression> useSet =getUseList((Statement)node);
			resultSet.setDefSet(definitionSet);
			resultSet.setUseSet(useSet);
		}
	
		return resultSet;
	}
		

	
	public void getDependencyListInIF(IfStatement ifNode)
	{
	
		ElementSet defuse = new ElementSet();
		CompoundStatement thenBody=(CompoundStatement)ifNode.getThenStatement();
		CompoundStatement elseBody=(CompoundStatement)ifNode.getElseStatement();
		
	    defuse=getDefUseSet(thenBody);
	    ElementSet tmpSet=getDefUseSet(elseBody);
		
	    defuse.getDefinitionSet().addAll(tmpSet.getDefinitionSet());
	    defuse.getUseSet().addAll(tmpSet.getUseSet());
	    defuse.getUseSet().addAll(getUseExpression(ifNode.getControlExpression()));	    
		DependencyPool.addDefUseSet(defuse,(Traversable) ifNode);
		
	}
	
	
	
	
	
	
	
	
	
	public ArrayList<Dependency>  getDependencyListInLoop(Loop currentLoop)
	{
		
		ArrayList<Dependency> dependencyList = new ArrayList<Dependency>();
		ElementSet defuse = new ElementSet();
	
		
		CompoundStatement body=(CompoundStatement)currentLoop.getBody();
		
		for(int i = 0; i<body.getChildren().size(); i++)
		{
			if( ! 	IgnoredNodes.isUninteresting(body.getChildren().get(i)))
			{
					for(int j = i+1; j<body.getChildren().size(); j++)		
							if(! IgnoredNodes.isUninteresting(body.getChildren().get(j)))
								dependencyList.addAll(getDependencyList(body.getChildren().get(i),body.getChildren().get(j)));				

			
			//calculating the DefUse Set for this loop 
			// Defuse = the union of the defuse of all statements inside the loop
				ElementSet firstSet= getDefUseSet(body.getChildren().get(i));
			
				if(!firstSet.getDefinitionSet().isEmpty())				
					for(int k= 0; k< firstSet.getDefinitionSet().size();k++)
							if(defuse.getDefinitionSet().indexOf(firstSet.getDefinitionSet().get(k))<0)
									defuse.getDefinitionSet().add(firstSet.getDefinitionSet().get(k));
				
				if(!firstSet.getUseSet().isEmpty())
				for(int k= 0; k< firstSet.getUseSet().size();k++)
					if(defuse.getUseSet().indexOf(firstSet.getUseSet().get(k))<0)
						defuse.getUseSet().add(firstSet.getUseSet().get(k));	
				
			}
			
		}
		
		DependencyPool.addDefUseSet(defuse, (Traversable)currentLoop);
		return dependencyList;
		
		
	}
	
	
	public ElementSet getDefUseSet(CompoundStatement body)
	{
		
		ElementSet defuse = new ElementSet();
		if(body==null)
		{  defuse.setDefSet(new ArrayList<Expression>());
			defuse.setUseSet(new ArrayList<Expression>());
			return defuse;
			
		}
		
		for(int i = 0; i<body.getChildren().size() && ! IgnoredNodes.isUninteresting(body.getChildren().get(i)); i++)
		{
			ElementSet firstSet= getDefUseSet(body.getChildren().get(i));
			if(!firstSet.getDefinitionSet().isEmpty())				
			for(int k= 0; k< firstSet.getDefinitionSet().size();k++)
				if(defuse.getDefinitionSet().indexOf(firstSet.getDefinitionSet().get(k))<0)
					defuse.getDefinitionSet().add(firstSet.getDefinitionSet().get(k));
			if(!firstSet.getUseSet().isEmpty())
			for(int k= 0; k< firstSet.getUseSet().size();k++)
				if(defuse.getUseSet().indexOf(firstSet.getUseSet().get(k))<0)
					defuse.getUseSet().add(firstSet.getUseSet().get(k));	
			
		
		}
		
		return defuse;
	}
	


	
	
	
	

}
