package ida.pelab.prt.annotation;

import ida.pelab.prt.generalfunctions.TraverseFunction;
import ida.pelab.prt.pattern.ArrayBoundary;
import ida.pelab.prt.pattern.LoopVariable;
import ida.pelab.prt.transformation.ExpressionSimplifier;

import java.util.ArrayList;

import cetus.hir.ArrayAccess;
import cetus.hir.BinaryExpression;
import cetus.hir.BinaryOperator;
import cetus.hir.BreadthFirstIterator;

import cetus.hir.Expression;

import cetus.hir.IntegerLiteral;
import cetus.hir.Traversable;


public class ArrayAnnotation  {

	


	public String GetAnnotationString(ArrayAccess arrayNodeTree) {
		
		String result =arrayNodeTree.getArrayName().toString();
		for(int i=0; i<arrayNodeTree.getIndices().size();i++)
		   result+= "[" + ExpressionSimplifier.Simplify( arrayNodeTree.getIndices().get(i)).toString()+"]";
		return result;
		
	}
	
	
	public static String toAnnotatedString(ArrayAccess inputArray,ArrayList<LoopVariable> variableList)
	{
		
		int j=0;
		String result =inputArray.getArrayName().toString();
		for(int i = 0 ; i < inputArray.getIndices().size();i++)
			{	
			  for( j= 0 ; j<variableList.size();j++)
				  if(variableList.get(j).equals(inputArray.getIndex(i))|| TraverseFunction.ExpressionContainsVariable( variableList.get(j).getVariableExpression(),inputArray.getIndex(i)))
					  break;
			
			  if(j<variableList.size())
				  	result+=anotateIndex(inputArray.getIndex(i), variableList.get(j));
			  
			  else    
				  result +="["+  ExpressionSimplifier.Simplify(inputArray.getIndex(i) ).toString()+"]";				
			}
		return result;
	}
	

	
	public static ArrayList<ArrayBoundary> toArrayBoundaryList(ArrayAccess inputArray,ArrayList<LoopVariable> variableList)
	{
		ArrayList<ArrayBoundary> indexBounaryList= new ArrayList<ArrayBoundary>();
		int j=0;

		for(int i = 0 ; i < inputArray.getIndices().size();i++)
			{	
			  for( j= 0 ; j<variableList.size();j++)
				  if(variableList.get(j).equals(inputArray.getIndex(i))|| TraverseFunction.ExpressionContainsVariable( variableList.get(j).getVariableExpression(),inputArray.getIndex(i)))
					  break;
			
			  if(j<variableList.size())
				  	 indexBounaryList.add(getAnnotatedIndex(inputArray.getIndex(i), variableList.get(j)));
			  

			}
		return indexBounaryList;
	}
	
	
	private static String anotateIndex(Expression index,LoopVariable variable )
	{
		
		if(variable.equals(index))
			return   	"[" + variable.toString()+"]";
		
		else 
		{
			Expression lowerBound=variable.getLowerBound();
			Expression upperBound =variable.getUpperBound();
			

			Expression lowerBoundinArray = ExpressionSimplifier.Simplify( getEquivalentExpression(index,variable,lowerBound));
			Expression upperBoundinArray = ExpressionSimplifier.Simplify( getEquivalentExpression(index,variable,upperBound));

			String stride = "";
			Expression indexStride =findStrideinArray(variable, index);
						
			if( (indexStride!=null && !indexStride.toString().equals("1")) || !variable.getStep().toString().equals("1"))
			{
				if(indexStride==null)
					stride =":" +ExpressionSimplifier.Simplify(variable.getStep()).toString();
				else
					{
						BinaryExpression mulEXP= new BinaryExpression(indexStride.clone(), BinaryOperator.MULTIPLY,variable.getStep().clone());
						stride =":"+ExpressionSimplifier.Simplify( mulEXP).toString();
					}
			}
			
			return "[" + lowerBoundinArray.toString()+":"+ upperBoundinArray.toString()+ stride+"]";
										
		}
		
		
	
	}

	private static ArrayBoundary getAnnotatedIndex(Expression index,LoopVariable variable )
	{
		
		if(variable.equals(index))
			return  new ArrayBoundary(
					ExpressionSimplifier.Simplify( variable.getLowerBound()).toString(),
					ExpressionSimplifier.Simplify(variable.getUpperBound()).toString(),
					ExpressionSimplifier.Simplify(variable.getStep()).toString()
					);

		else 
		{
			Expression lowerBound=variable.getLowerBound();
			Expression upperBound =variable.getUpperBound();
			

			Expression lowerBoundinArray = ExpressionSimplifier.Simplify( getEquivalentExpression(index,variable,lowerBound));
			Expression upperBoundinArray = ExpressionSimplifier.Simplify( getEquivalentExpression(index,variable,upperBound));

			String stride = "";
			Expression indexStride =findStrideinArray(variable, index);
						
			if( (indexStride!=null && !indexStride.toString().equals("1")) || !variable.getStep().toString().equals("1"))
			{
				if(indexStride==null)
					stride =ExpressionSimplifier.Simplify(variable.getStep()).toString();
				else
					{
						BinaryExpression mulEXP= new BinaryExpression(indexStride.clone(), BinaryOperator.MULTIPLY,variable.getStep().clone());
						stride =ExpressionSimplifier.Simplify( mulEXP).toString();
					}
			}
			
			return new ArrayBoundary( lowerBoundinArray.toString(), upperBoundinArray.toString(), stride);
										
		}
		
		
	
	}
	
	
	// gets the indices of the array and replaces another expression with the matching expression 
	private static Expression  getEquivalentExpression(Expression array,LoopVariable variable, Expression boundary)
	{
		Expression tmpArray =array.clone();
		BreadthFirstIterator<Traversable>  bfsIterator = new BreadthFirstIterator<Traversable>(tmpArray);
		while(bfsIterator.hasNext())
		{
			Object currentNode = bfsIterator.next();
			if(variable.equals(currentNode))
			{		
				((Expression)currentNode).swapWith(boundary.clone());
				//break;
			};
			 
				
		}
		
		return  tmpArray;
			
		
	}
	
	
	private static Expression findStrideinArray(LoopVariable variable, Expression arrayIndex)
	{
		
		Expression indexStride=null;
		BreadthFirstIterator<Traversable>  bfsIterator = new BreadthFirstIterator<Traversable>(arrayIndex);
		while(bfsIterator.hasNext())
		{
			Object currentNode = bfsIterator.next();
			if(variable.equals(currentNode))
			{
				Traversable parentNode =((Traversable)currentNode).getParent();
				if(parentNode instanceof BinaryExpression )
				{
					BinaryExpression  binExp =(BinaryExpression)parentNode;
					if(binExp.getOperator().equals(BinaryOperator.MULTIPLY))
					{
						if(binExp.getLHS().equals(variable.getVariableExpression()))
							indexStride=binExp.getRHS();
						else indexStride=binExp.getLHS();
						break;
						
					}
					else if(binExp.getOperator().equals(BinaryOperator.SUBTRACT))
					{
						if(binExp.getRHS().equals(variable.getVariableExpression()))
							indexStride= new BinaryExpression(new IntegerLiteral(0), BinaryOperator.SUBTRACT, new IntegerLiteral(1));
						else indexStride=new IntegerLiteral(1);
						break;
						
					}
					
				}
			
			
				
			}
				
		
		}
		
	
		
		return  indexStride;
		
	}

}
