package ida.pelab.prt.generalfunctions;


import java.util.List;


import ida.pelab.prt.transformation.ExpressionSimplifier;


import cetus.hir.ArrayAccess;
import cetus.hir.BinaryExpression;
import cetus.hir.BinaryOperator;

import cetus.hir.Expression;
import cetus.hir.ExpressionStatement;
import cetus.hir.ForLoop;
import cetus.hir.Identifier;
import cetus.hir.IntegerLiteral;

import cetus.hir.AssignmentExpression;
import cetus.hir.AssignmentOperator;
import cetus.hir.NullStatement;
import cetus.hir.PostOrderIterator;
import cetus.hir.Statement;

import cetus.hir.Traversable;
import cetus.hir.UnaryExpression;
import cetus.hir.UnaryOperator;


public class LoopFunctions {
	
	public static Expression  getLoopvariable(ForLoop fLoop)
	{
		

		if(fLoop.getCondition() instanceof BinaryExpression)
		{
			BinaryExpression conditionExpression =(BinaryExpression)fLoop.getCondition();
		 	Expression loopVariable =conditionExpression.getLHS();
		 	if((loopVariable instanceof Identifier)||(loopVariable instanceof ArrayAccess))
		 		return loopVariable ;
		 	else return null;
		 	// if  i+3=5 can happen
		 	//if(loopVariable instanceof Identifier)
		 		
		 //	else return null;
		}
		else return null;
	}
	
	
	public static BinaryOperator  getLoopOperator(ForLoop fLoop)
	{
		
		if(fLoop.getCondition() instanceof BinaryExpression)
		{
			BinaryExpression conditionExpression =(BinaryExpression)fLoop.getCondition();
			BinaryOperator loopOperator = conditionExpression.getOperator();
		 	
		 	return loopOperator ;
		}
		else return null;
	}

	
	public  static LoopDirectionType detectLoopDirection(ForLoop fLoop)
	{
	  	BinaryOperator loopOP = getLoopOperator(fLoop);
	  	if(loopOP.equals(BinaryOperator.COMPARE_LE)|| loopOP.equals(BinaryOperator.COMPARE_LT))
	  		return LoopDirectionType.Ascending;
	  	
	  	else if(loopOP.equals(BinaryOperator.COMPARE_GT)|| loopOP.equals(BinaryOperator.COMPARE_GE))
	  		return LoopDirectionType.Descending;
		
	  	else return LoopDirectionType.Unkown;
	}
	

	
	public static Expression getLowerBoundExpression(ForLoop fLoop)
	{
		Expression resultExpression =null;
		Statement initialStatement=fLoop.getInitialStatement();		
		if(initialStatement instanceof NullStatement)
			return  getLoopvariable(fLoop).clone();
		
		Expression initExp =((ExpressionStatement)initialStatement).getExpression();
		if(initExp instanceof BinaryExpression)
			resultExpression= ((BinaryExpression)initExp).getRHS().clone();
		
		return resultExpression;
	}
	
	

	public static Expression getUpperBoundExpression(ForLoop lp)
	{
	

		if( lp.getCondition() instanceof BinaryExpression)
		{
			BinaryExpression binExp=(BinaryExpression)lp.getCondition();
			Expression result=binExp.getRHS().clone();
			
			if(binExp.getOperator().equals(BinaryOperator.COMPARE_LT))
				result =new BinaryExpression(result, BinaryOperator.SUBTRACT, new IntegerLiteral(1));
			else if (binExp.getOperator().equals(BinaryOperator.COMPARE_GT))
				result =new BinaryExpression(result, BinaryOperator.ADD, new IntegerLiteral(1));
			return result;
			
		}
		else return null;
	
		
	}
	
	//
	public static Expression getUpperBoundmax(ForLoop lp)
	{
	

		if( lp.getCondition() instanceof BinaryExpression)
		{
			BinaryExpression binExp=(BinaryExpression)lp.getCondition();
			Expression result=binExp.getRHS().clone();
			
			if(binExp.getOperator().equals(BinaryOperator.COMPARE_LT) || binExp.getOperator().equals(BinaryOperator.COMPARE_GT))
			return result;			
		}
		 return null;
	
		
	}
	
	public static  String convertBoundToString(Expression boundExpression)
	{
		if(boundExpression!=null)
			return boundExpression.toString();
		
		return "";
	}

	//returns the increment value added to the loop variable in the step expression
	public static Expression getStepIncrement(ForLoop fLoop)
	{
		Expression stepExpression =findStep(fLoop);
		
		// covers i++,++i,i--,--i
		if(stepExpression instanceof UnaryExpression )			
			{
			    if(  ((UnaryExpression)stepExpression).getOperator().equals(UnaryOperator.POST_INCREMENT)
			    ||  ((UnaryExpression)stepExpression).getOperator().equals(UnaryOperator.PRE_INCREMENT))			    			    	
			    	return new IntegerLiteral(1);
			    
			    else if(((UnaryExpression)stepExpression).getOperator().equals(UnaryOperator.POST_DECREMENT)
			    ||  ((UnaryExpression)stepExpression).getOperator().equals(UnaryOperator.PRE_DECREMENT))
			     return new IntegerLiteral(-1);	
			    
			    else return null ;
			}
		
		//covers i+= ,i-=
		else if(stepExpression instanceof AssignmentExpression && 
				!((AssignmentExpression)stepExpression).getOperator().equals(AssignmentOperator.NORMAL))
		{
			AssignmentOperator op =((AssignmentExpression)stepExpression).getOperator();
			if(op.equals(AssignmentOperator.SUBTRACT))	
			   //return  new  BinaryExpression(new IntegerLiteral(0), BinaryOperator.SUBTRACT, (( AssignmentExpression) stepExpression).getRHS().clone() );
			return new UnaryExpression(UnaryOperator.MINUS, (( AssignmentExpression) stepExpression).getRHS().clone());
			   else	return   (( AssignmentExpression) stepExpression).getRHS().clone();
			
			
		}
		
		//covers i = i+expression ,i=i-expression  
		// replaces the i on the rightSide with 0
		else if(stepExpression instanceof BinaryExpression)
		{
			
			Expression leftside=((BinaryExpression)stepExpression).getLHS();
			//leftside.setParent(stepExpression);
			Expression rightside=((BinaryExpression)stepExpression).getRHS().clone();
		//	rightside.setParent(stepExpression);
			PostOrderIterator<Traversable> rightSideIterator = new PostOrderIterator<Traversable>(rightside);
			while(rightSideIterator.hasNext())
			{
				Object currentNode = rightSideIterator.next();
				if(currentNode instanceof Expression && ((Expression)currentNode).equals(leftside))								
					((Expression)currentNode).swapWith( new IntegerLiteral(0));								
	
			}
			return  rightside;
	
		}	
		
		return null;
		
	}

	
	
	// finds the actual expression that represents the step
	public static Expression findStep(ForLoop fLoop)
	{
		
		Expression loopVariable =LoopFunctions.getLoopvariable(fLoop);
		 if(loopVariable==null)
			 return null ;
		
		 Expression tmpStep=fLoop.getStep();
		 if(tmpStep!=null)
			  if(isStepValid(loopVariable,tmpStep))
				  return tmpStep ;
			  else return null ;
		 else 
		 {
			 
			 List<Traversable>  bodychildren = fLoop.getBody().getChildren();
			 for(int i= 0; i< bodychildren.size();i++)
				 {
				 	Traversable child= bodychildren.get(i);
				 	if(child instanceof ExpressionStatement && isStepValid(loopVariable, ((ExpressionStatement)child).getExpression()))
				 	{
				 		     Expression loopExp=((ExpressionStatement)child).getExpression();	 		   		 		    		  
				 		     return 	loopExp;						 
				 	}
				 	
				 }
			   	return null;
			 
			 
			 
		 }
		 
	}
	
	
	//checks whether the input step is the actual step of the loop
	public static boolean isStep(ForLoop fLoop , Traversable inputExp)
	{
	   /*	Expression loopExpr = findStep(fLoop);
		
			if(loopExpr!=null)
				return  (loopExpr.equals(inputExp) ) && (System.identityHashCode(loopExpr)== System.identityHashCode(inputExp)) ||
				
		*/
		
		Expression loopExpr = findStep(fLoop);
		  if(loopExpr!=null)
		    {
			 
			  Expression simplifiedExp =ExpressionSimplifier.Simplify(loopExpr);
			  if(loopExpr.getParent()==inputExp.getParent() &&  loopExpr.equals(inputExp))
				  return true ;
			  if(loopExpr.getParent()==inputExp.getParent() &&  simplifiedExp.equals(inputExp))
				  return true ;
			  return false ;
			  
		    }
			
			
		
			
			return false ;
			
		
		
	}
	
	
	
	// the step of the loop is considered as valid if it contains the loopvariable on the left side as a single variable
	public static boolean isStepValid(Expression loopVariable ,Expression stepExression)
	{
		
		if(stepExression instanceof BinaryExpression)
		{
			BinaryExpression binExpression =(BinaryExpression)stepExression;
		 	Expression leftSide =binExpression.getLHS();
		 	return  leftSide.equals(loopVariable);
		 	
		}
		
		else if(stepExression instanceof UnaryExpression)
		{
			UnaryExpression unExpression=(UnaryExpression)stepExression;
			if(!loopVariable.equals(unExpression.getExpression()))
					return false ;
			else return true ;
		}
		
		 return false;
		
		
	}
	
	
	public static Expression getConditionUpperbound(Expression condition,Expression variable)
	{
	
		Expression upperBound=null;
		if(condition instanceof BinaryExpression)	
	    {
	       BinaryExpression binExp=(BinaryExpression) condition;
	       // only the <,<= operators are valid
	       
	       if(( binExp.getOperator().equals(BinaryOperator.COMPARE_LE) || binExp.getOperator().equals(BinaryOperator.COMPARE_LT)) 
	    	&& binExp.getLHS().equals(variable))	      
	    	   upperBound=binExp.getRHS();
	    }
		
		return upperBound;
		
	}
	
	
	public static boolean isForLoopValid(ForLoop forLoop)
	{
		return (forLoop.getCondition()!=null) && (LoopFunctions.findStep(forLoop)!=null)
				&& ( LoopFunctions.getLoopvariable(forLoop)!=null) &&
					(LoopFunctions.getLoopOperator(forLoop) !=null);					
	}
	
	
	}
