package ida.pelab.prt.patterndefinition;

import ida.pelab.prt.constraints.ArrayIndexCheck;
import ida.pelab.prt.constraints.BoundaryCheck;
import ida.pelab.prt.constraints.Constraint;
import ida.pelab.prt.constraints.IFStructureCheck;
import ida.pelab.prt.constraints.IndexRuleCheck;
import ida.pelab.prt.constraints.LoopCheck;
import ida.pelab.prt.constraints.MemoryAccess;
import ida.pelab.prt.constraints.MemoryAccessType;
import ida.pelab.prt.constraints.NameCheck;
import ida.pelab.prt.constraints.RuleCheck;
import ida.pelab.prt.constraints.RuleCheckOperator;
import ida.pelab.prt.constraints.RuleParameterType;
import ida.pelab.prt.constraints.TypeCheck;
import ida.pelab.prt.generalfunctions.LoopDirectionType;
import ida.pelab.prt.pattern.PatternLevel;
import ida.pelab.prt.pattern.PatternOutputFormat;
import ida.pelab.prt.pattern.PatternStructure;
import ida.pelab.prt.pattern.PatternStructureOperator;
import ida.pelab.prt.pattern.PatternType;
import ida.pelab.prt.pattern.PrintArgument;
import ida.pelab.prt.pattern.PrintArgumentType;
import ida.pelab.prt.patternhierarchy.PatternHierarchyGraph;

import java.io.File;

import java.net.URLDecoder;

import java.util.ArrayList;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;


public class PatternParser {

	private static DocumentBuilderFactory dbfbBuilder;
	private static File xmlFile;
	private static ArrayList<PatternStructure> patternObjects = new ArrayList<PatternStructure>();

	private static boolean loadXmlFile() {
		try {
			dbfbBuilder = DocumentBuilderFactory.newInstance();

			String packagePath = PatternParser.class.getResource(
					"PatternParser.class").getPath();
			String databasePath = packagePath.substring(0, packagePath
					.lastIndexOf("/") + 1)
					+ "PatternDefinition.xml";
			String externalPath = System.getProperty("user.dir") + "/"
					+ "PatternDefinition.xml";
			xmlFile = new File(externalPath);
			if (!xmlFile.exists()) {
				xmlFile = new File(URLDecoder.decode(databasePath, "UTF-8"));
				if (!xmlFile.exists()) {
					System.out.println("Can't find the PatternDefinition.xml file in ");
					System.out.println(externalPath);
					return false;
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}

		return true;

	}

	public static boolean parse() {
		try {

			if (!PatternHierarchyGraph.isEmpty())
				return true;
			if (!loadXmlFile())
				return false;

			DocumentBuilder builder = dbfbBuilder.newDocumentBuilder();
			Document patternDoc = builder.parse(xmlFile);

			NodeList patternList = patternDoc.getElementsByTagName("pattern");
			for (int i = 0; i < patternList.getLength(); i++) {

				Node currentElement = patternList.item(i);
				if (currentElement.getNodeType() == Node.ELEMENT_NODE) {
					Element pattern = (Element) currentElement;
					String name = pattern.getAttribute("name");
					boolean helpFlag = false;
					if (pattern.hasAttribute("helppattern"))
						helpFlag = true;
					String typeString = pattern.getElementsByTagName("type")
							.item(0).getTextContent();
					PatternType patternType = PatternType.valueOf(
							PatternType.class, typeString);
					String patternLevelString = pattern.getElementsByTagName(
							"level").item(0).getTextContent();
					PatternLevel patternLevel = PatternLevel
							.valueOf(patternLevelString);

					PatternStructure newPattern = getsubStructure(pattern);
					newPattern.setName(name);
					newPattern.setHelpPattern(helpFlag);
					newPattern.setLevel(patternLevel);
					newPattern.setType(patternType);
					patternObjects.add(newPattern);
					PatternHierarchyGraph.addPatternToGraph(newPattern);

				}
			}

			updatePHGReferences();
			return true;
		} catch (Exception e) {
			// TODO: handle exception
			e.printStackTrace();
			return false;
		}
	}

	private static PatternStructure getsubStructure(Element pattern) {
		ArrayList<ArrayList<String>> structureArrayString = new ArrayList<ArrayList<String>>();
		PatternStructureOperator op = null;
		boolean hasCustomRules=false;
		boolean skipCondition=false;
		ArrayList<Constraint> constraints = new ArrayList<Constraint>();             
		Element structureElm = seekXMLtag(pattern, "structure");          
		if (structureElm != null) {
			
			String customStructureCheck= structureElm.getAttribute("hascustommatchingrules");
			if(customStructureCheck.equals("true"))
				hasCustomRules=true;
			structureElm = seekXMLtag(pattern, "tree");
			String operatorString = structureElm.getAttribute("operator");
			if (operatorString.equals(""))
				op = PatternStructureOperator.NONE;
			else
				op = PatternStructureOperator.valueOf(operatorString);

			Element childrenelm= seekXMLtag(structureElm, "children");
			if(childrenelm!=null)
			{
				String conditionSkipFlag= childrenelm.getAttribute("skipcondition");
				if(conditionSkipFlag.equals("true"))
					skipCondition=true;
			}
			NodeList nodes = structureElm.getElementsByTagName("child");
			for (int j = 0; j < nodes.getLength(); j++) {
				ArrayList<String> tmpArray = new ArrayList<String>();
				Element tmpElement = (Element) nodes.item(j);
				if (tmpElement.getNodeType() == Node.ELEMENT_NODE) {
					NodeList elementType = tmpElement
							.getElementsByTagName("type");
					for (int k = 0; k < elementType.getLength(); k++) {
						Element tmpType = (Element) elementType.item(k);
						if (tmpType.getNodeType() == Node.ELEMENT_NODE) {
							String context = tmpType.getTextContent();
							tmpArray.add(context);
						}

					}

					structureArrayString.add(tmpArray);
				}

			}

			structureElm = seekXMLtag(structureElm, "constraints");
			if (structureElm != null) {
				constraints.addAll(getTypeCheckRules(structureElm));
				constraints.addAll(getNameCheckRules(structureElm));
				constraints.addAll(getRuleCheckRules(structureElm));
				constraints.addAll(getBoundaryCheckRules(structureElm));
				constraints.addAll(getMemoryAccess(structureElm));
				constraints.addAll(getIFStructure(structureElm));
				constraints.addAll(getArrayIndexStructure(structureElm));
				constraints.addAll(getForLoopCheckRule(structureElm));
			}

		}

		PatternOutputFormat output = getOutputFormat(pattern);

		PatternStructure ptrStructure = new PatternStructure();
		ptrStructure.setOperator(op);
		ptrStructure.setCustomMatchingRules(hasCustomRules);
		ptrStructure.setPruneIFCondtion(skipCondition);
		ptrStructure.setStructureElement(structureArrayString);
		ptrStructure.setConstraints(constraints);
		if (output == null)
			output = new PatternOutputFormat();
		ptrStructure.setOutputSyntax(output);

		return ptrStructure;

	}

	private static PatternOutputFormat getOutputFormat(Element elm) {
		ArrayList<PrintArgument> printArg = new ArrayList<PrintArgument>();
		NodeList structureNode = elm.getElementsByTagName("output");
		if (structureNode.getLength() == 0)
			return new PatternOutputFormat();

		String printingName = getContentValue(elm, "printingname");
		Element innerNode = seekXMLtag(elm, "args");

		NodeList checkNodes = innerNode.getElementsByTagName("parameter");
		for (int j = 0; j < checkNodes.getLength(); j++) {
			Element tmpElement = (Element) checkNodes.item(j);
			if (tmpElement.getNodeType() == Node.ELEMENT_NODE) {
				String typeString = tmpElement.getAttribute("type");
				PrintArgumentType type = PrintArgumentType.valueOf(typeString);
				String value = tmpElement.getAttribute("value");
				printArg.add(new PrintArgument(type, value));
			}
		}

		return new PatternOutputFormat(printingName, printArg);

	}

	private static ArrayList<Constraint> getBoundaryCheckRules(Element elm) {
		ArrayList<Constraint> resultSet = new ArrayList<Constraint>();
		NodeList checkNodes = elm.getElementsByTagName("boundarycheck");
		for (int j = 0; j < checkNodes.getLength(); j++) {
			Element tmpElement = (Element) checkNodes.item(j);
			if (tmpElement.getNodeType() == Node.ELEMENT_NODE) {
				String paramID = tmpElement.getAttribute("paramid");
				String startIndex = tmpElement.getAttribute("startindex");
				String endIndex = tmpElement.getAttribute("endindex");
				BoundaryCheck boundaryChecking = new BoundaryCheck(paramID,
						startIndex, endIndex);
				resultSet.add(boundaryChecking);
			}
		}

		return resultSet;

	}

	private static ArrayList<Constraint> getMemoryAccess(Element elm) {
		ArrayList<Constraint> resultSet = new ArrayList<Constraint>();
		NodeList checkNodes = elm.getElementsByTagName("nomemoryaccess");
		for (int j = 0; j < checkNodes.getLength(); j++) {
			Element tmpElement = (Element) checkNodes.item(j);
			if (tmpElement.getNodeType() == Node.ELEMENT_NODE) {
				String paramID = tmpElement.getAttribute("paramid");
				String accessTypeString = tmpElement.getAttribute("type");
				MemoryAccessType accessType = MemoryAccessType
						.valueOf(accessTypeString);
				String startPoint = tmpElement.getAttribute("startpoint");
				String endPoint = tmpElement.getAttribute("endpoint");
				MemoryAccess memoryAccess = new MemoryAccess(accessType,
						paramID, startPoint, endPoint);
				resultSet.add(memoryAccess);
			}
		}
		return resultSet;

	}

	private static ArrayList<Constraint> getArrayIndexStructure(Element elm) {
		ArrayList<Constraint> resultSet = new ArrayList<Constraint>();
		ArrayIndexCheck indexCheck = null;
		// Element indexElm =seekXMLtag(elm,"indexstructure");
		NodeList indexnodes = elm.getElementsByTagName("indexstructure");
		for (int i = 0; i < indexnodes.getLength(); i++) {
			Element indexElement = (Element) indexnodes.item(i);
			if (indexElement.getNodeType() == Node.ELEMENT_NODE) {
				indexCheck = new ArrayIndexCheck();
				String paramID = indexElement.getAttribute("paramid");
				indexCheck.setParameterID(paramID);
				Element indicesElm = seekXMLtag(indexElement, "indices");
				if (indicesElm != null) {
					NodeList nodes = indicesElm.getElementsByTagName("index");
					for (int j = 0; j < nodes.getLength(); j++) {
						Element tmpElement = (Element) nodes.item(j);
						if (tmpElement.getNodeType() == Node.ELEMENT_NODE) {
							NodeList typenodes = tmpElement
									.getElementsByTagName("type");
							ArrayList<String> typeVar = new ArrayList<String>();
							for (int w = 0; w < typenodes.getLength(); w++) {
								Element typeElement = (Element) typenodes
										.item(w);
								if (typeElement.getNodeType() == Node.ELEMENT_NODE) {
									String type = typeElement.getTextContent();
									typeVar.add(type);

								}
							}
							indexCheck.AddIndex(typeVar);

						}
					}
				}
			}
			ArrayList<Constraint> rules = getIndexRuleCheckRules(indexElement);
			for (int k = 0; k < rules.size(); k++)
				indexCheck.addIndexconstraint((IndexRuleCheck) rules.get(k));

			resultSet.add(indexCheck);
		}

		return resultSet;

	}

	private static ArrayList<Constraint> getIFStructure(Element elm) {
		ArrayList<Constraint> resultSet = new ArrayList<Constraint>();
		IFStructureCheck ifconstraint = new IFStructureCheck();

		Element ifElm = seekXMLtag(elm, "ifblock");
		if (ifElm != null) {
			Element thenElm = seekXMLtag(elm, "thenblock");
			if (thenElm != null) {
				NodeList nodes = thenElm.getElementsByTagName("statement");
				for (int j = 0; j < nodes.getLength(); j++) {
					Element tmpElement = (Element) nodes.item(j);
					if (tmpElement.getNodeType() == Node.ELEMENT_NODE) {
						String paramID = tmpElement.getAttribute("paramid");
						ifconstraint.AddtoThenStructure(paramID);
					}
				}
			}
			Element elseElm = seekXMLtag(elm, "elseblock");
			if (elseElm != null) {
				NodeList nodes = elseElm.getElementsByTagName("statement");
				for (int j = 0; j < nodes.getLength(); j++) {
					Element tmpElement = (Element) nodes.item(j);
					if (tmpElement.getNodeType() == Node.ELEMENT_NODE) {
						String paramID = tmpElement.getAttribute("paramid");
						ifconstraint.AddtoELseStructure(paramID);
					}
				}
			}

			resultSet.add(ifconstraint);
		}
		return resultSet;

	}

	private static ArrayList<Constraint> getTypeCheckRules(Element elm) {
		ArrayList<Constraint> resultSet = new ArrayList<Constraint>();
		NodeList checkNodes = elm.getElementsByTagName("typecheck");
		for (int j = 0; j < checkNodes.getLength(); j++) {
			Element tmpElement = (Element) checkNodes.item(j);
			if (tmpElement.getNodeType() == Node.ELEMENT_NODE) {
				String paramID = tmpElement.getAttribute("paramid");
				String paramtypeString = tmpElement.getAttribute("type");
				RuleParameterType ruleParam = RuleParameterType.valueOf(
						RuleParameterType.class, paramtypeString);
				TypeCheck typeChecking = new TypeCheck(ruleParam, paramID);
				resultSet.add(typeChecking);
			}
		}
		return resultSet;

	}

	private static ArrayList<Constraint> getNameCheckRules(Element elm) {
		ArrayList<Constraint> resultSet = new ArrayList<Constraint>();
		NodeList checkNodes = elm.getElementsByTagName("namecheck");
		for (int j = 0; j < checkNodes.getLength(); j++) {
			Element tmpElement = (Element) checkNodes.item(j);
			if (tmpElement.getNodeType() == Node.ELEMENT_NODE) {
				String paramID = tmpElement.getAttribute("paramid");
				String value = tmpElement.getAttribute("value");
				NameCheck typeChecking = new NameCheck(value, paramID);
				resultSet.add(typeChecking);
			}
		}
		return resultSet;

	}

	private static ArrayList<Constraint> getForLoopCheckRule(Element elm) {
		ArrayList<Constraint> resultSet = new ArrayList<Constraint>();
		Element forElm = seekXMLtag(elm, "forloopcheck");
		if (forElm != null) {
			String stepInc = forElm.getAttribute("stepincrement");
			String dir = forElm.getAttribute("direction");
			LoopDirectionType dirType;
			if (dir.equals("ASC"))
				dirType = LoopDirectionType.Ascending;
			else if (dir.equals("DESC"))
				dirType = LoopDirectionType.Descending;
			else
				dirType = LoopDirectionType.Unkown;
			LoopCheck lc = new LoopCheck(stepInc, dirType);
			resultSet.add(lc);
		}
		return resultSet;

	}

	private static ArrayList<Constraint> getIndexRuleCheckRules(Element elm) {
		ArrayList<Constraint> resultSet = new ArrayList<Constraint>();
		NodeList checkNodes = elm.getElementsByTagName("indexrulecheck");
		for (int j = 0; j < checkNodes.getLength(); j++) {

			ArrayList<String> paramList = new ArrayList<String>();
			Element tmpElement = (Element) checkNodes.item(j);
			if (tmpElement.getNodeType() == Node.ELEMENT_NODE) {
				String ruleOperatorString = tmpElement.getAttribute("type");
				RuleCheckOperator ruleOP = RuleCheckOperator
						.valueOf(ruleOperatorString);

				NodeList elementType = tmpElement.getElementsByTagName("arg");
				for (int k = 0; k < elementType.getLength(); k++) {
					Element tmpType = (Element) elementType.item(k);
					if (tmpType.getNodeType() == Node.ELEMENT_NODE) {
						String arg = tmpType.getAttribute("paramid");
						paramList.add(arg);
					}

				}
				IndexRuleCheck newRule = new IndexRuleCheck(ruleOP, paramList);
				resultSet.add(newRule);
			}
		}
		return resultSet;

	}

	private static ArrayList<Constraint> getRuleCheckRules(Element elm) {
		ArrayList<Constraint> resultSet = new ArrayList<Constraint>();
		NodeList checkNodes = elm.getElementsByTagName("rulecheck");
		for (int j = 0; j < checkNodes.getLength(); j++) {

			ArrayList<String> paramList = new ArrayList<String>();
			Element tmpElement = (Element) checkNodes.item(j);
			if (tmpElement.getNodeType() == Node.ELEMENT_NODE) {
				String ruleOperatorString = tmpElement.getAttribute("type");
				RuleCheckOperator ruleOP = RuleCheckOperator
						.valueOf(ruleOperatorString);

				NodeList elementType = tmpElement.getElementsByTagName("arg");
				for (int k = 0; k < elementType.getLength(); k++) {
					Element tmpType = (Element) elementType.item(k);
					if (tmpType.getNodeType() == Node.ELEMENT_NODE) {
						String arg = tmpType.getAttribute("paramid");
						paramList.add(arg);
					}

				}
				RuleCheck newRule = new RuleCheck(ruleOP, paramList);
				resultSet.add(newRule);
			}
		}
		return resultSet;

	}

	private static Element seekXMLtag(Element xmlElement, String tagName) {
		NodeList structureNode = xmlElement.getElementsByTagName(tagName);
		return (Element) structureNode.item(0);

	}

	private static String getContentValue(Element xmlElement, String tagName) {

		return xmlElement.getElementsByTagName(tagName).item(0)
				.getTextContent();

	}

	private static void updatePHGReferences() {
		for (int i = 0; i < patternObjects.size(); i++) {
			PatternStructure currentPattern = patternObjects.get(i);
			ArrayList<ArrayList<String>> sprtStructure = currentPattern
					.getStructureElement();
			for (int j = 0; j < sprtStructure.size(); j++)
				for (int k = 0; k < sprtStructure.get(j).size(); k++) {
					PatternStructure lowLevel = getPatternByName(
							patternObjects, sprtStructure.get(j).get(k));
					if (lowLevel != null)
						PatternHierarchyGraph.AddTopLevelPattern(lowLevel,
								currentPattern);
					else if(sprtStructure.get(j).get(k).equals("ALLLEVEL0") || sprtStructure.get(j).get(k).equals("ALLLEVEL1")
							|| sprtStructure.get(j).get(k).equals("ALLLEVEL2"))
						extendTopLevelPatterns(currentPattern, sprtStructure.get(j).get(k),patternObjects);
				}

		}

	}

	private static PatternStructure getPatternByName(
			ArrayList<PatternStructure> patternList, String name) {

		for (int i = 0; i < patternList.size(); i++)
			if (patternList.get(i).getName().equals(name))
				return patternList.get(i);

		return null;
	}
	
	
	private static void extendTopLevelPatterns(PatternStructure currentPattern,String subStrucString,ArrayList<PatternStructure>  patternObjectList)
	{
   
		ArrayList<PatternStructure> lowLevelCandidate =getPatternByLevel(patternObjects, subStrucString);
		if(lowLevelCandidate!=null)
			for(int i=0;i<lowLevelCandidate.size();i++)
				PatternHierarchyGraph.AddTopLevelPattern(lowLevelCandidate.get(i),
				currentPattern);
	
	}
  private static ArrayList<PatternStructure> getPatternByLevel(ArrayList<PatternStructure> ptrList, String Level)
  {
	     PatternLevel level=null;
	     if(Level.equals("ALLLEVEL0"))
	    	 level=PatternLevel.Level0;
	     else if (Level.equals("ALLLEVEL1"))
	    	 level=PatternLevel.Level1;
	     else if(Level.equals("ALLLEVEL2"))
	    	 level=PatternLevel.Level2;
	     else return null;
	     
	     ArrayList<PatternStructure>  patternLevelList=new ArrayList<PatternStructure>();
	     for(int i=0; i< ptrList.size();i++)
	    	 if(ptrList.get(i).getLevel().equals(level))
	    	 patternLevelList.add(ptrList.get(i));
	     
	     return patternLevelList;
  
  }

}
