package ida.pelab.prt.patternhierarchy;


import ida.pelab.prt.pattern.PatternLevel;
import ida.pelab.prt.pattern.PatternStructure;
import ida.pelab.prt.pattern.PatternType;

import java.util.ArrayList;
import java.util.Hashtable;




public class PatternHierarchyGraph {
	
	
	private static  Hashtable<PatternStructure, ArrayList<PatternStructure>>  patternGraph=new Hashtable<PatternStructure, ArrayList<PatternStructure>>();
	private static  Hashtable<PatternLevel,ArrayList<PatternStructure>>  orderedPatternGraph=new Hashtable<PatternLevel, ArrayList<PatternStructure>>();
	private static  ArrayList<PatternStructure>  HorizontalPatternList=new ArrayList<PatternStructure>();
	private  PatternHierarchyGraph()
	{
		patternGraph = new Hashtable<PatternStructure, ArrayList<PatternStructure>>();
		orderedPatternGraph = new Hashtable<PatternLevel, ArrayList<PatternStructure>>();
	}
	
	public static ArrayList<PatternStructure>  getTopLevelPattern(PatternStructure pattern )
	{
		if(pattern==null)
			return null;
		
		if(patternGraph.containsKey(pattern))
		    if(pattern.getLevel().equals(PatternLevel.Trivial))
		    	return getPatternsByLevel(PatternLevel.Level0);
		    else
		    	return patternGraph.get(pattern);
		else 
			return null;
	}
	
  public static void  addPatternToGraph(PatternStructure newPattern,ArrayList<PatternStructure> topLevelPatterns)
  {
	  ArrayList<PatternStructure> currentLevelPattern=null;
	 // System.out.print(patternGraph==null);
	  patternGraph.put(newPattern, topLevelPatterns);
	  if(orderedPatternGraph.containsKey(newPattern.getLevel()))
	  {
		  currentLevelPattern = orderedPatternGraph.get(newPattern.getLevel());
		  currentLevelPattern.add(newPattern);
	  }
	  else
	  {
		  currentLevelPattern = new ArrayList<PatternStructure>();
		  currentLevelPattern.add(newPattern);
		  orderedPatternGraph.put(newPattern.getLevel(), currentLevelPattern);
	  }
	  
	  if(newPattern.getType().equals(PatternType.Horizontal))
		  HorizontalPatternList.add(newPattern);
	  
  }
  
  
  // for the patterns that  have no toplevel pattern
  public static void  addPatternToGraph(PatternStructure newPattern)
  {
	  ArrayList<PatternStructure> currentLevelPattern=null;
	  ArrayList<PatternStructure>  emptyTopLevel = new ArrayList<PatternStructure>();
	 // System.out.print(patternGraph==null);
	  patternGraph.put(newPattern, emptyTopLevel);
	  if(orderedPatternGraph.containsKey(newPattern.getLevel()))
	  {
		  currentLevelPattern = orderedPatternGraph.get(newPattern.getLevel());
		  currentLevelPattern.add(newPattern);
	  }
	  else
	  {
		  currentLevelPattern = new ArrayList<PatternStructure>();
		  currentLevelPattern.add(newPattern);
		  orderedPatternGraph.put(newPattern.getLevel(), currentLevelPattern);
	  }
	  if(newPattern.getType().equals(PatternType.Horizontal))
		  HorizontalPatternList.add(newPattern);
	  
  }
  
  
  public static ArrayList<PatternStructure>  getPatternsByLevel(PatternLevel level)
  {
	  if(orderedPatternGraph.containsKey(level))
	  
		 return orderedPatternGraph.get(level);
	  else 
		  return null;
	  
  }
  

 /**
  * checks whether there exists a pattern with the name "patternName" at the level specified by the level variable
 * @param patternName
 * @param level
 * @return
 */
public static boolean hasPatternatLevel(String patternName,PatternLevel level)
 {
	 ArrayList<PatternStructure>  patternList= getPatternsByLevel(level);
	 for(int i=0; i< patternList.size();i++)
		 if(patternList.get(i).getName().equals(patternName))
	 		return true;
	 
	 		
	 	return false;	
 }

  
  public static void AddTopLevelPattern(PatternStructure currentPattern,PatternStructure topLevelPattern)
  {
	  ArrayList<PatternStructure> topLevel =getTopLevelPattern(currentPattern );
	  if(topLevel==null || topLevel.contains(topLevelPattern))
		  return;
	  topLevel.add(topLevelPattern);
	  
  }
  
  public  static  ArrayList<PatternStructure> getHorizontalList()
  {
	  
	  return HorizontalPatternList;

  }
  
  

  
  
  public static  boolean isEmpty()
  {
	  return  !( patternGraph.size()>0 || HorizontalPatternList.size()>0); 
	  
	  
  }
  
  public static void  clear()
  {
	
	  patternGraph.clear();
	  orderedPatternGraph.clear();
	  HorizontalPatternList.clear();
	  
	  
  }
  
  
}
