package dsptester;

import ida.pelab.prt.analysis.RecognitionAnalysis;
import ida.pelab.prt.temporalAnalysis.TemporalAnalysis;

import java.io.File;

import java.security.Permission;
import java.text.NumberFormat;
import java.util.ArrayList;



import report.ReportData;
import report.ReportGenerator;


public class TestcaseRunner {
	
 private String path;
 
 public TestcaseRunner()
 {
	 setPath("");
 }

public void setPath(String path) {
	this.path = path;
}

public String getPath() {
	return path;
}
	

private static ArrayList<Double> forLoopDetectionInfo = new ArrayList<Double>();
public static ArrayList<Double> getStatementDetectionInfo() {
	return statementDetectionInfo;
}

public static ArrayList<Double> getForLoopDetectionInfo() {
	return forLoopDetectionInfo;
}


private static ArrayList<Double> statementDetectionInfo = new ArrayList<Double>();

public static void main(String[] args)

{
	if(args.length<1)
		{
			System.err.println("Please enter the path for the folder containing the C files:");
			return ;
   		}
	 
	TestcaseRunner programTester = new TestcaseRunner();
	programTester.setPath(args[0]);
	  
	SecurityManager securityManager = new SecurityManager() {
		@Override
	    public void checkPermission(Permission permission) {
	        if (permission.getName().indexOf("exitVM")>-1) {
	        
	        	System.err.println("program is trying to exit");
	               throw new SecurityException("System.exit attempted and blocked.");
	        }
	    }
	};
	System.setSecurityManager(securityManager);

	
	File sourceDirectory = new File(programTester.getPath());

	
	ArrayList<ReportData> data = new ArrayList<ReportData>();
	
	String [] fileList = sourceDirectory.list(new FileNameSyntax());
	if(fileList==null)
	{
		System.err.println("The Path of the Source folder is not valid");
		return;
	}
	for(int i= 0; i<fileList.length;i++)
	{
	   ReportData rdt = new ReportData();	
		
		
		File tmpfile = new File(fileList[i]);		
			
		    rdt.setFileName(tmpfile.getName());
		    rdt.setSize(Long.toString( tmpfile.getAbsoluteFile().length()));
		   
			System.out.println("running the tool on " +tmpfile.getName() + "("+ Integer.toString(i+1)+"/" +  Integer.toString(fileList.length)+ ")" );
			String [] programArgs =  new String[2]; 
			programArgs[0]="-idaprt=";						
			programArgs[1]=	 sourceDirectory+ "/"+tmpfile.getName();
			
			try{
			cetus.exec.Driver.main(programArgs);
			
			NumberFormat percentageFormat ;
		 	percentageFormat=NumberFormat.getInstance();
		 	percentageFormat.setMaximumFractionDigits(2);
			rdt.setStatementCount( Double.toString( RecognitionAnalysis.getStatementCount()));
			
			rdt.setDetectedPattern( Double.toString(RecognitionAnalysis.getDetectedPatterns()));
			double stmtCount=RecognitionAnalysis.getStatementCount();
			String stmtCoverage=percentageFormat.format(RecognitionAnalysis.getStatementCoverage()*100)+   "%";
			if(stmtCount==0)
				stmtCoverage = "<STRIKE>" + stmtCoverage + "</STRIKE>";
			else statementDetectionInfo.add(RecognitionAnalysis.getStatementCoverage());
			rdt.setDetectedStatmentPercentage(stmtCoverage );
			
			String detectedLoopPercentage = percentageFormat.format(RecognitionAnalysis.getForLoopCoverage()*100)+   "%"  ;
			String detectedLoopResult = Double.toString(RecognitionAnalysis.getDetectedForloopCNT()) + " out of " + Double.toString (RecognitionAnalysis.getLoopCount());
			if(RecognitionAnalysis.getLoopCount()==0)
				{
				detectedLoopPercentage= "<STRIKE>" + detectedLoopPercentage + "</STRIKE>";
				detectedLoopResult= "<STRIKE>" + detectedLoopResult + "</STRIKE>";
				}
			else forLoopDetectionInfo.add(RecognitionAnalysis.getForLoopCoverage());
			rdt.setDetectedForloop(detectedLoopResult);
			rdt.setDetectedForloopPercentage(detectedLoopPercentage);			
			rdt.setValidCodePercentage(percentageFormat.format(RecognitionAnalysis.getValidCodeRatio()*100)+"%");
			rdt.setRecogntionTime(TemporalAnalysis.getRecognitionTime());
			
			
			rdt.setTransformationTime(TemporalAnalysis.getTransformationTime());
			rdt.setDetectedPointersInStatements(percentageFormat.format(RecognitionAnalysis.getPointerCoverage()*100)+ "%");	
			ReportGenerator.printNonDetectedLevel0Patterns(tmpfile.getName(), RecognitionAnalysis.getNonDetectedStatements());
			rdt.setErrorStatement("");
			}
			catch (Exception e) {
				// TODO: handle exception
				if(! (e instanceof SecurityException))
					e.printStackTrace();
				rdt.setErrorStatement("Error parsing the file");
				
			}
			finally
			{
				data.add(rdt);
				
				
			}
			
	
		
		
		
	}
	

		ReportGenerator.generateReport(data);

		System.out.println("program finished successfully ");
}


}


